/*
  Example allows remote control of Led Diode connected to pin 1.
  To access device, download LocalServer from https://stargate-system.com
  and run it on machine within the same network.

  Example ready to use with Raspberry Pico W.
  To use with other boards, uncomment WiFi library include relevant to your board

  Before uploading, replace <YOUR WIFI NAME> and <YOUR WIFI PASSWORD>
  with your WiFi credentials
*/

// Uncomment below if using ESP-8266
// #include <ESP8266WiFi.h>

// Uncomment below if using ESP-32
// #include <WiFi.h>

#include <WiFiGateDevice.h>

int ledPin = 1;

// creating device instance
WiFiGateDevice device("<YOUR WIFI NAME>", "<YOUR WIFI PASSWORD>");
// declaring Gate Value to store current LED setting
GateInt led = device.factory.createInt("input");

void setup() {
  pinMode(ledPin, OUTPUT);
  // setting device name
  device.setName("Basic LED");
  // setting Gate Value name
  led.setName("Brightness");
  // setting allowed range
  led.setMin(0);
  led.setMax(255);
  // setting callback to be invoked if Gate Value was changed remotely
  led.onRemoteChange = onLedChange;
  // starting device
  device.start();
}

void loop() {
  // running device internal processes
  device.loop();
}

void onLedChange() {
  analogWrite(ledPin, led.getValue());
}
