/*
  Example of observing device state and value subscription state

  Led Diode connected to GP1 pin behaves as follows:
    - no connection with LocalServer - led off;
    - device connected, observedValue not subscribed - blinks
    - observedValue subscribed - led on

  Usage:
    1. Upload sketch to your board - led should be off
    2. Start LocalServer - led should blik indicating successful connection
      (some boards needs restart after uploading program)
    3. Open LocalServer in browser - led should be on
      (page subscribes all visible values)
    4. Close browser - led should blink
      (values are unsubscribed on closing page)
    5. Stop LocalServer - led should be off indicating connection lost

  Example ready to use with Raspberry Pico W.
  To use with other boards, uncomment WiFi library include relevant to your board

  Before uploading, replace <YOUR WIFI NAME> and <YOUR WIFI PASSWORD>
  with your WiFi credentials
*/

// Uncomment below if using ESP-8266
// #include <ESP8266WiFi.h>

// Uncomment below if using ESP-32
// #include <WiFi.h>

#include <WiFiGateDevice.h>

int ledPin = 1;

// Creating device instance
WiFiGateDevice device("<YOUR WIFI NAME>", "<YOUR WIFI PASSWORD>");
// Defining dummy value to observe it's subscription state
GateBool observedValue = device.factory.createBool("output");

bool ledState = false;

void setup() {
  pinMode(ledPin, OUTPUT);
  device.setName("State Observer");
  device.start();
}

void loop() {
  // running device internal processes
  device.loop();
  // Checking connection state
  if (device.isReady()) {
    // Checking subscription state
    if (observedValue.subscribed) {
      digitalWrite(ledPin, HIGH);
    } else {
      ledState = !ledState;
      digitalWrite(ledPin, ledState);
      delay(200);
    }
  } else {
    digitalWrite(ledPin, LOW);
  }
}